// MDView.cpp : implementation of the CMDView class
//

#include "stdafx.h"
#include "MD.h"

#include "MDDoc.h"
#include "MDView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const long WINDOW_BUFFER_RESOLUTION=10000;	// Matt Kuntz's way of drawing atoms with more resolution than screen

#include "Rand.h"
#include "RandomPositionStrategy.h"
#include "ThermalizeVelocityStrategy.h"
#include "UpdateStrategy.h"

UINT simThread(LPVOID pointer)
{

	CMDView* atomView =  (CMDView *) pointer;
	
	CMDDoc* pDoc = atomView->GetDocument();
	MDSimulation *sim = &(pDoc->sim);
	Rand rand;

	sim->nonOverlapRandom->Move(sim->atoms);

	sim->thermalize->Move(sim->atoms);

	double t=0;

	while (!atomView->stopThread)
	{
		t += sim->update->GetDt();
		for (int steps=0; steps<sim->timeStepsBetweenObservations; steps++)
		{
			if (pDoc->pauseThread)
			{
				pDoc->threadPaused.SetEvent();	// Acknowledge thread pause
				pDoc->paramsChanged.Lock();		// Wait for parameters to change
				pDoc->paramsChanged.Unlock();	// Silly Microsoft convention
				pDoc->pauseThread=false;
			}
			sim->update->Move(sim->atoms);
		}
		atomView->AtomsMoved(t);
		Sleep(0);	// Improves responsiveness
	}

	atomView->stopThread=false;
	atomView->threadDone.SetEvent();	// Notify main thread of shutdown

	return 1;
}

/////////////////////////////////////////////////////////////////////////////
// CMDView

IMPLEMENT_DYNCREATE(CMDView, CView)

BEGIN_MESSAGE_MAP(CMDView, CView)
	//{{AFX_MSG_MAP(CMDView)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMDView construction/destruction

CMDView::CMDView()
{
	buffer = NULL;	// Set up in OnInitialUpdate
	stored_pDC = NULL;	// Set up first time used in AtomsMoved (later Update), then store
	stopThread=false;
}

CMDView::~CMDView()
{
	if (!stopThread)		// Thread still running?
	{
		stopThread=TRUE;	// Ask thread to shut down
		threadDone.Lock();	// Wait for thread to announce shut down with SetEvent
		threadDone.Unlock();	// Meaningless Windows convention: analogy to semaphore according to Matt Kuntz
	}

	delete buffer;
	if (stored_pDC != NULL && ::IsWindow(m_hWnd)) ReleaseDC(stored_pDC);	// from AtomsMoved (later Update)
}

BOOL CMDView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMDView drawing

void CMDView::OnDraw(CDC* pDC)
{

	CMDDoc* pDoc = GetDocument();
//	ASSERT_VALID(pDoc);	// XXX If not commented out, this causes the program to fail
						// This is because we're drawing to the screen from another thread
						// (If we were drawing from another PROCESS, Matt Kuntz says that this would really fail)
						// We really should attach a safe Windows handle to the CMDView, and 
						// when calling from another thread call a subroutine that generates the CWin structure
						// from that thread. I'm confused about how that would work.

	// From Matt Kuntz's program, Spring 1998

	Atoms *atoms = pDoc->sim.atoms;

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(WINDOW_BUFFER_RESOLUTION,WINDOW_BUFFER_RESOLUTION);	// Resolution of window
	pDC->SetViewportExt(window_width,-window_height);	// Reverse direction of y
	pDC->SetViewportOrg(0,window_height);	// Set 0 to LLHS

	// clear screen

	bufferDC.PatBlt(0,0,WINDOW_BUFFER_RESOLUTION,WINDOW_BUFFER_RESOLUTION,WHITENESS);

	// draw atoms in position

	// XXX For three dimensions, should sort by z-coordinate!

	for(long i=0; i<atoms->Number(); i++)
	{
		double x = atoms->X(i,0);
		double y = atoms->X(i,1);
		bufferDC.Ellipse((int)((x-atoms->GetRadius())*dX), (int)((y-atoms->GetRadius())*dY),
			(int)((x+atoms->GetRadius())*dX), (int)((y+atoms->GetRadius())*dY));
	}

	// Put buffer to window

	pDC->BitBlt(0,0,WINDOW_BUFFER_RESOLUTION,WINDOW_BUFFER_RESOLUTION,&bufferDC,0,0,SRCCOPY);

}

/////////////////////////////////////////////////////////////////////////////
// CMDView printing

BOOL CMDView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CMDView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CMDView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CMDView diagnostics

#ifdef _DEBUG
void CMDView::AssertValid() const
{
	CView::AssertValid();
}

void CMDView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CMDDoc* CMDView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMDDoc)));
	return (CMDDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMDView message handlers

void CMDView::AtomsMoved(double t)
{
	if(!::IsWindow(m_hWnd)) return;
	if (stored_pDC==NULL) stored_pDC = GetDC();	// For AtomsMoved (later Update)
	OnDraw(stored_pDC);
}

void CMDView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();

	CMDDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	double width = pDoc->sim.atoms->BoxSize()[0];
	double height = pDoc->sim.atoms->BoxSize()[1];

	CClientDC dc(this);


	CRect window_rect;		// Find size of window to draw to
	GetClientRect(&window_rect);


	window_height = window_rect.Height();
	window_width = window_rect.Width();

	bufferDC.CreateCompatibleDC(&dc);

	buffer = new CBitmap;

	buffer->CreateCompatibleBitmap(&dc,window_width,window_height);
	oldBuffer = bufferDC.SelectObject(buffer);

	bufferDC.SetMapMode(MM_ISOTROPIC);
	bufferDC.SetWindowExt(WINDOW_BUFFER_RESOLUTION,WINDOW_BUFFER_RESOLUTION);       // Resolution of window
	bufferDC.SetViewportExt(window_width,-window_height);   // Reverse direction of y
	bufferDC.SetViewportOrg(0,window_height);       // Set 0 to LLHS

	bufferDC.PatBlt(0,0,WINDOW_BUFFER_RESOLUTION,WINDOW_BUFFER_RESOLUTION,WHITENESS);

	dX = WINDOW_BUFFER_RESOLUTION/width;    // Coordinate conversion factor
	dY = WINDOW_BUFFER_RESOLUTION/height;

	bufferDC.SelectStockObject(BLACK_PEN);
	bufferDC.SelectStockObject(GRAY_BRUSH);

//	AfxBeginThread(simThread,this->m_hWnd);
	AfxBeginThread(simThread,this);

}

